#include <stdlib.h>
#include <new>
#include <limits>
#include <string.h>
#include <iomanip>
#include <iostream>
#include <vector>
#include <list>

class TStaticBehavior
{
public:
	TStaticBehavior()
	{
		std::cout << "TStaticBehavior Constructor" << std::endl;
	}

	~TStaticBehavior()
	{
		std::cout << "TStaticBehavior Destructor" << std::endl;
	}

};


template <class TDType> class TDSTLAllocator
{
public:
	typedef size_t size_type;
	typedef ptrdiff_t difference_type;
	typedef TDType * pointer;
	typedef const TDType * const_pointer;
	typedef TDType & reference;
	typedef const TDType & const_reference;
	typedef TDType value_type;

	template <class TDOther> struct rebind { typedef TDSTLAllocator<TDOther> other; };

	static bool & GetOverWrite()
	{
		static bool overWrite = true;
		return overWrite;
	}

	static int & GetIndex()
	{
		static TStaticBehavior b;
		static int index = 0x7fffffff;
		return index;
	}

public:

	pointer address(reference v) const
	{
		return (&v);
	}

	const_pointer address(const_reference v) const
	{
		return (&v);
	}

	TDSTLAllocator() throw() {}

	TDSTLAllocator(const TDSTLAllocator<TDType> & src) throw()
	{
		if(GetOverWrite() || GetIndex() == 0x7fffffff)
		{
			std::cout << "constructior0 this : " << (void *)this << ", old : " << GetIndex() << ", new : " << src.GetIndex() << ", pt = " << &GetIndex() << std::endl;
			GetIndex() = src.GetIndex();
		}
	}

	template<class TDOther> 
		TDSTLAllocator(const TDSTLAllocator<TDOther> & src) throw() 
	{
		if(GetOverWrite() || GetIndex() == 0x7fffffff)
		{
			std::cout << "constructior1 this : " << (void *)this << ", old : " << GetIndex() << ", new : " << src.GetIndex() << ", pt = " << &GetIndex() << std::endl;
			GetIndex() = src.GetIndex();
		}
	}

	TDSTLAllocator& operator=( const TDSTLAllocator& ha )
	{
		if(GetOverWrite() || GetIndex() == 0x7fffffff)
		{
			std::cout << "substitute this : " << (void *)this << ", old : " << GetIndex() << ", new : " << src.GetIndex() << ", pt = " << &GetIndex() << std::endl;
			GetIndex() = ha.GetIndex();
		}
		return *this;
	}

	pointer allocate(size_type _Count, const_pointer _Hint = 0)
	{
		unsigned char * tp;
		tp = new unsigned char[_Count * sizeof(TDType)];

		//monitoring
		std::cout << "allocate this : " << (void *)this << ", index : " << GetIndex() << ", pt = " << &GetIndex() << std::endl;

		//
		return (pointer)tp;
	}

	void construct(pointer _Ptr, const TDType & _Value)
	{
		new((void*)_Ptr) TDType(_Value);
	}

	void deallocate(pointer _Ptr, size_type _Count)
	{
		//monitoring
		std::cout << "deallocate this : " << (void *)this << ", index : " << GetIndex() << ", pt = " << &GetIndex() << std::endl;


		//
		delete [] (unsigned char *)_Ptr;
	}

	void destroy(pointer _Ptr)
	{
		_Ptr->~TDType();
	}

	size_type max_size() const throw()
	{
		return std::numeric_limits<size_t>::max() / sizeof(TDType);
	}


};

template <typename T1> bool operator==( const TDSTLAllocator<T1>& lhs, const TDSTLAllocator<T1>& rhs) throw()
{
	return (bool)true;
}

template <typename T1> bool operator!=( const TDSTLAllocator<T1>& lhs, const TDSTLAllocator<T1>& rhs) throw()
{
	return (bool)false;
}

struct TA
{
	short a;
};

struct TB
{
	float b;
};

void AllocatorTypeTest()
{
	TDSTLAllocator<TA> a0;
	TDSTLAllocator<TB> a1;

	a0.GetIndex() = 2;
	a1.GetIndex() = 3;

	std::vector<TA, TDSTLAllocator<TA> > v0(a0);
	std::list<TB, TDSTLAllocator<TB> > v1(a1);

	v0.resize(5);
	//v1.resize(6);

	v1.push_back(TB());

	std::vector<TA, TDSTLAllocator<TA> > v2(a0);
	v2.resize(7);

	//std::_Aux_cont
}

void InitAuxCont()
{
#ifdef _HAS_ITERATOR_DEBUGGING
#if !_HAS_ITERATOR_DEBUGGING
	TDSTLAllocator<std::_Aux_cont> a0;
	TDSTLAllocator<std::_Aux_cont>::GetIndex() = 1;
	TDSTLAllocator<std::_Aux_cont>::GetOverWrite() = false;
#endif
#endif
}


int main()
{
	InitAuxCont();

	AllocatorTypeTest();
	
	return 0;
}